use thiserror::Error;

use crate::parse;

#[derive(Debug, Error)]
#[error("missing field {0:?}")]
pub struct MissingFieldError(pub String);

#[derive(Debug, Error)]
#[error("bad field data: {0}")]
pub struct FieldDataError(#[source] pub parse::Error<parse::InputOwned>);

impl<'a> From<parse::Error<parse::Input<'a>>> for FieldDataError {
    fn from(err: parse::Error<parse::Input<'a>>) -> Self {
        FieldDataError(err.into_owned())
    }
}

impl<'a> From<nom::Err<parse::Error<parse::Input<'a>>>> for FieldDataError {
    fn from(err: nom::Err<parse::Error<parse::Input<'a>>>) -> Self {
        match err {
            nom::Err::Error(e) | nom::Err::Failure(e) => e.into(),
            nom::Err::Incomplete(_) => unreachable!("incomplete error"),
        }
    }
}
